@@ -35,7 +35,7 @@
 struct Archive*
 newArchive (void)
 {
-  return calloc (sizeof (struct Archive), 1);
+  return calloc (1, sizeof (struct Archive));
 }
 /** Deallocate an archive data structure.
  * @param archive       the archive to be deallocated
@@ -82,9 +82,10 @@ WriteArchive (const struct Filename* name,
   }
 
   /* check for duplicate file names */
-  for (ae = archive->first; ae; ae = ae->next)
-    if (!memcmp (&ae->name.name, name->name, sizeof name->name))
-      return WrFileExists;
+  if (!allowDuplicates)
+    for (ae = archive->first; ae; ae = ae->next)
+      if (!memcmp (&ae->name.name, name->name, sizeof name->name))
+        return WrFileExists;
 
   if (!(ae = malloc (sizeof (*ae)))) {
     (*log) (Errors, name, "Out of memory.");
