@@ -6,7 +6,7 @@
  */
 
 /*
-** Copyright © 1993-1998,2001,2006,2021,2022 Marko Mäkelä
+** Copyright © 1993-1998,2001,2006,2021,2022,2024 Marko Mäkelä
 ** 1581 disk image management by Pasi Ojala
 **
 **     This program is free software; you can redistribute it and/or modify
@@ -315,7 +315,7 @@ isFreeBlock (const struct Image* image, byte_t track, byte_t sector)
 
       return true && BAM[(track * 3) + (sector >> 3)] & (1 << (sector & 7));
     }
-    /* fall through: track <= 35 */
+    /* fall through */
   case Im1541:
     if (!(BAM = getBlock ((struct Image*) image, image->dirtrack, 0)))
       return false;
@@ -616,7 +616,7 @@ allocBlock (struct Image* image,
       findNextFree (image, track, sector);
       return true;
     }
-    /* fall through: track <= 35 */
+    /* fall through */
   case Im1541:
     if (!(BAM = getBlock ((struct Image*) image, image->dirtrack, 0)))
       return false;
@@ -638,18 +638,16 @@ allocBlock (struct Image* image,
     {
       byte_t** BAMblocks = 0;
       byte_t offset;
+      size_t s;
 
-      if(*track > image->partTops[image->dirtrack - 1] ||
-         *track < image->partBots[image->dirtrack - 1])
+      if (*track > image->partTops[image->dirtrack - 1] ||
+          *track < image->partBots[image->dirtrack - 1])
         return false;
 
-      if (2 != mapInode (&BAMblocks, image, image->dirtrack, 1, 0, 0)) {
-        if (BAMblocks) free (BAMblocks);
-        return false;
-      }
+      s = mapInode (&BAMblocks, image, image->dirtrack, 1, 0, 0);
 
       offset = *track;
-      if(offset > 40) {
+      if (offset > 40) {
         BAM = BAMblocks[1];
         offset -= 40;
       }
@@ -658,6 +656,9 @@ allocBlock (struct Image* image,
 
       free (BAMblocks);
 
+      if (2 != s)
+        return false;
+
       if (!(BAM[16 + (offset - 1) * 6 + (*sector >> 3) + 1] &
             (1 << (*sector & 7))))
         return false; /* already allocated */
@@ -980,20 +981,17 @@ backupBAM (const struct Image* image, byte_t** BAM)
   case Im1581:
     {
       byte_t** bamblocks = 0;
+      bool ok = 2 == mapInode (&bamblocks, (struct Image*) image,
+                               image->dirtrack, 1, 0, 0) &&
+        !!(*BAM = malloc (2 << 8));
 
-      if (2 != mapInode (&bamblocks, (struct Image*) image,
-                         image->dirtrack, 1, 0, 0)) {
-        if (bamblocks) free (bamblocks);
-        return false;
+      if (ok) {
+        memcpy (*BAM, bamblocks[0], 256);
+        memcpy (*BAM + 256, bamblocks[1], 256);
       }
 
-      if (!(*BAM = malloc (2 << 8)))
-        return false;
-
-      memcpy (*BAM, bamblocks[0], 256);
-      memcpy (*BAM + 256, bamblocks[1], 256);
       free (bamblocks);
-      return true;
+      return ok;
     }
   }
 
@@ -1050,7 +1048,7 @@ restoreBAM (struct Image* image, byte_t** BAM)
       byte_t** bamblocks = 0;
 
       if (2 != mapInode (&bamblocks, image, image->dirtrack, 1, 0, 0)) {
-        if (bamblocks) free (bamblocks);
+        free (bamblocks);
         return false;
       }
 
@@ -1160,7 +1158,7 @@ freeBlock (struct Image* image, byte_t track, byte_t sector)
       BAM2[((tr - 1) * 3) + (sector >> 3)] |= (byte_t) (1 << (sector & 7));
       return true;
     }
-    /* fall through: track <= 35 */
+    /* fall through */
   case Im1541:
     if (!(BAM = getBlock ((struct Image*) image, image->dirtrack, 0)))
       return false;
@@ -1174,19 +1172,16 @@ freeBlock (struct Image* image, byte_t track, byte_t sector)
   case Im1581:
     {
       byte_t** BAMblocks = 0;
+      size_t s;
 
-      if(track > image->partTops[image->dirtrack - 1] ||
-         track < image->partBots[image->dirtrack - 1])
+      if (track > image->partTops[image->dirtrack - 1] ||
+          track < image->partBots[image->dirtrack - 1])
         return false;
 
-      if (2 != mapInode (&BAMblocks, image, image->dirtrack, 1, 0, 0)) {
-        if (BAMblocks) free (BAMblocks);
-        return false;
-      }
+      s = mapInode (&BAMblocks, image, image->dirtrack, 1, 0, 0);
 
-      if(track > 40) {
+      if (track > 40) {
         BAM = BAMblocks[1];
-
         track -= 40;
       }
       else
@@ -1194,6 +1189,9 @@ freeBlock (struct Image* image, byte_t track, byte_t sector)
 
       free (BAMblocks);
 
+      if (2 != s)
+        return false;
+
       BAM[16 + (track - 1) * 6] += 1;
       BAM[16 + (track - 1) * 6 + (sector >> 3) + 1] |= (byte_t) (1 << (sector & 7));
       return true;
@@ -1294,7 +1292,8 @@ getDirEnt (struct Image* image,
         freeslotEntry = i;
       }
 
-      if (!memcmp (dirent[i].name, name->name, 16)) {
+      if (image->direntOpts < DirEntDupCreate &&
+          !memcmp (dirent[i].name, name->name, 16)) {
         free (directory);
         return &dirent[i];
       }
@@ -1369,13 +1368,13 @@ getDirEnt (struct Image* image,
   /* Clear the directory entry. */
 
   dirent = &((struct DirEnt*) directory[freeslotBlock])[freeslotEntry];
+  free (directory);
 
   if (freeslotEntry)
     memset (dirent, 0, sizeof *dirent);
   else
     memset (&dirent->type, 0, (sizeof *dirent) - 2);
 
-  free (directory);
   return dirent;
 }
 
@@ -1546,24 +1545,20 @@ blocksFree (const struct Image* image)
     {
       byte_t** BAMblocks = 0;
 
-      if (2 != mapInode (&BAMblocks, (struct Image*) image,
+      if (2 == mapInode (&BAMblocks, (struct Image*) image,
                          image->dirtrack, 1, 0, 0)) {
-        if (BAMblocks) free (BAMblocks);
-        return 0;
+        for (track = image->partBots[image->dirtrack - 1];
+             track <= image->partTops[image->dirtrack - 1] && track <= 40;
+             track++)
+          sum += BAMblocks[0][16 + (track - 1)*6];
+
+        for (track = image->partTops[image->dirtrack - 1];
+             track >= image->partBots[image->dirtrack - 1] && track > 40;
+             track--)
+          sum += BAMblocks[1][16 + (track - 41) * 6];
       }
 
-      for (track = image->partBots[image->dirtrack - 1];
-           track <= image->partTops[image->dirtrack - 1] && track <= 40;
-           track++)
-        sum += BAMblocks[0][16 + (track - 1)*6];
-
-      for (track = image->partTops[image->dirtrack - 1];
-           track >= image->partBots[image->dirtrack - 1] && track > 40;
-           track--)
-        sum += BAMblocks[1][16 + (track - 41) * 6];
-
       free (BAMblocks);
-
       return sum;
     }
   }
@@ -1585,13 +1580,14 @@ setupSideSectors (struct Image* image,
                   log_t log)
 {
   size_t sscount;
+  enum WrStatus status;
 
 #ifdef DEBUG
   if (!isValidDirEnt (image, dirent) || getFiletype (image, dirent) != REL)
     return WrFail;
 #endif
 
-  if(image->type == Im1581)
+  if (image->type == Im1581)
     return WrFail; /* TODO: super side sector setup etc. */
 
   sscount = rounddiv (blocks, 120);
@@ -1608,7 +1604,6 @@ setupSideSectors (struct Image* image,
 
   {
     byte_t* buf;
-    enum WrStatus status;
     size_t sslength = 14 + 254 * (sscount - 1) + 2 * (blocks % 120);
 
     if (!(buf = calloc (sslength, 1)))
@@ -1617,64 +1612,61 @@ setupSideSectors (struct Image* image,
     status = writeInode (image, dirent->ssTrack, dirent->ssSector,
                          buf, sslength);
     free (buf);
-
-    if (status != WrOK)
-      return status;
   }
 
-  {
+  if (status == WrOK) {
     byte_t** sidesect = 0;
     byte_t** datafile = 0;
-    size_t ssentry;
-    byte_t ss, i, track, sector;
+    status = WrFail;
 
-    if (blocks != mapInode (&datafile, image,
+    if (blocks == mapInode (&datafile, image,
                             dirent->firstTrack, dirent->firstSector,
-                            log, dirent))
-      return WrFail;
-
-    if (sscount != mapInode (&sidesect, image,
+                            log, dirent) &&
+        sscount == mapInode (&sidesect, image,
                              dirent->ssTrack, dirent->ssSector,
                              log, dirent)) {
-      free (datafile);
-      return WrFail;
-    }
+      size_t ssentry;
+      byte_t ss, i, track, sector;
+      for (ss = 0; ss < sscount; ss++) {
+        sidesect[ss][2] = ss;
+        sidesect[ss][3] = dirent->recordLength;
 
-    for (ss = 0; ss < sscount; ss++) {
-      sidesect[ss][2] = ss;
-      sidesect[ss][3] = dirent->recordLength;
+        sidesect[ss][4] = dirent->ssTrack;
+        sidesect[ss][5] = dirent->ssSector;
 
-      sidesect[ss][4] = dirent->ssTrack;
-      sidesect[ss][5] = dirent->ssSector;
-
-      for (i = 1; i < sscount; i++) {
-        sidesect[ss][4 + i * 2] = sidesect[i - 1][0];
-        sidesect[ss][5 + i * 2] = sidesect[i - 1][1];
+        for (i = 1; i < sscount; i++) {
+          sidesect[ss][4 + i * 2] = sidesect[i - 1][0];
+          sidesect[ss][5 + i * 2] = sidesect[i - 1][1];
+        }
       }
+
+      track = dirent->firstTrack;
+      sector = dirent->firstSector;
+
+      for (ssentry = 0; track; ssentry++) {
+        if (ssentry / 120 >= sscount)
+          goto Done;
+
+        ss = (byte_t) (ssentry / 120);
+        sidesect[ss][16 + (ssentry % 120) * 2] = track;
+        sidesect[ss][17 + (ssentry % 120) * 2] = sector;
+
+        track = datafile[ssentry][0];
+        sector = datafile[ssentry][1];
+      }
+
+      status = WrOK;
     }
 
-    track = dirent->firstTrack;
-    sector = dirent->firstSector;
-
-    for (ssentry = 0; track; ssentry++) {
-      if (ssentry / 120 >= sscount)
-        return WrFail;
-
-      ss = (byte_t) (ssentry / 120);
-      sidesect[ss][16 + (ssentry % 120) * 2] = track;
-      sidesect[ss][17 + (ssentry % 120) * 2] = sector;
-
-      track = datafile[ssentry][0];
-      sector = datafile[ssentry][1];
-    }
-
+  Done:
     free (datafile);
     free (sidesect);
   }
 
-  return WrOK;
+  return status;
 }
 
+
 /** Check if the side sectors of a relative file are OK
  * @param image         the disk image
  * @param dirent        the directory entry
@@ -1690,6 +1682,7 @@ checkSideSectors (const struct Image* image,
   byte_t** sidesect = 0;
   byte_t** datafile = 0;
   byte_t track, sector;
+  bool ok = false;
 
 #ifdef DEBUG
   if (!isValidDirEnt (image, dirent))
@@ -1702,25 +1695,17 @@ checkSideSectors (const struct Image* image,
   /* Map the data file and the side sectors. */
 
   if (!(i = mapInode (&datafile, (struct Image*) image,
-                      dirent->firstTrack, dirent->firstSector, log, dirent)))
-    return false;
-
-  if (!(sscount = mapInode (&sidesect, (struct Image*) image,
+                      dirent->firstTrack, dirent->firstSector, log, dirent)) ||
+      !(sscount = mapInode (&sidesect, (struct Image*) image,
                             dirent->ssTrack, dirent->ssSector,
-                            log, dirent))) {
-    free (datafile);
-    return false;
-  }
+                            log, dirent)))
+    goto Done;
 
   /* Check the block counts */
   if (sscount != rounddiv(i, 120) ||
       i + sscount != dirent->blocksLow + ((unsigned) dirent->blocksHigh << 8) ||
-      i != 120U * (sscount - 1U) + (sidesect[sscount - 1][1] - 15) / 2U) {
-  Failed:
-    free (datafile);
-    free (sidesect);
-    return false;
-  }
+      i != 120U * (sscount - 1U) + (sidesect[sscount - 1][1] - 15) / 2U)
+    goto Done;
 
   /* Check the side sector links */
 
@@ -1729,12 +1714,12 @@ checkSideSectors (const struct Image* image,
         sidesect[ss][3] != dirent->recordLength ||
         sidesect[ss][4] != dirent->ssTrack ||
         sidesect[ss][5] != dirent->ssSector)
-      goto Failed;
+      goto Done;
 
     for (i = 1; i < sscount; i++)
       if (sidesect[ss][4 + i * 2] != sidesect[i - 1][0] ||
           sidesect[ss][5 + i * 2] != sidesect[i - 1][1])
-        goto Failed;
+        goto Done;
   }
 
   /* Check the links to the data file */
@@ -1748,16 +1733,17 @@ checkSideSectors (const struct Image* image,
     if (ss >= sscount ||
         sidesect[ss][16 + (ssentry % 120) * 2] != track ||
         sidesect[ss][17 + (ssentry % 120) * 2] != sector)
-      goto Failed;
+      goto Done;
 
     track = datafile[ssentry][0];
     sector = datafile[ssentry][1];
   }
 
+  ok = true;
+ Done:
   free (datafile);
   free (sidesect);
-
-  return true;
+  return ok;
 }
 
 /** Generate a CP/M translation table.
@@ -1905,27 +1891,21 @@ WriteCpmImage (const struct Filename* name,
 {
   byte_t** trans;
   struct CpmDirEnt** allocated;
-  struct CpmDirEnt* dirent;
+  struct CpmDirEnt* dirent = 0;
   struct CpmDirEnt cpmname;
   unsigned au; /* allocation unit size */
   unsigned sectors; /* number of disk sectors */
   unsigned slot; /* next directory slot */
   unsigned blocksfree; /* number of free blocks */
+  enum WrStatus status = WrFail;
 
   if (!name || !data || !image || !image->buf ||
       !(trans = CpmTransTable (image, &au, &sectors)))
     return WrFail;
 
-  if (!(allocated = calloc (2 * sectors / au, sizeof(*allocated)))) {
-    free (trans);
-    return WrFail;
-  }
-
-  if (!(dirent = malloc (au * 8 * sizeof (*dirent)))) {
-    free (allocated);
-    free (trans);
-    return WrFail;
-  }
+  if (!(allocated = calloc (2 * sectors / au, sizeof(*allocated))) ||
+      !(dirent = malloc (au * 8 * sizeof (*dirent))))
+    goto Done;
 
   blocksfree = 2 * (sectors / au - 1);
 
@@ -1993,13 +1973,12 @@ WriteCpmImage (const struct Filename* name,
           !memcmp (&dirent[d], "\0\0\0\0\0\0\0\0\0\0\0", 12))
         continue;
 
-      if (!memcmp (dirent[d].basename, cpmname.basename,
+      if (image->direntOpts < DirEntDupCreate &&
+          !memcmp (dirent[d].basename, cpmname.basename,
                    sizeof cpmname.basename + sizeof cpmname.suffix)) {
-        if (image->direntOpts == DirEntOnlyCreate) {
-          free (dirent);
-          free (allocated);
-          free (trans);
-          return WrFileExists;
+        if (image->direntOpts == DirEntUniqCreate) {
+          status = WrFileExists;
+          goto Done;
         }
 
         found = true;
@@ -2033,12 +2012,8 @@ WriteCpmImage (const struct Filename* name,
     }
 
     /* See if the file was found */
-    if (!found && image->direntOpts == DirEntDontCreate) {
-      free (dirent);
-      free (allocated);
-      free (trans);
-      return WrFail;
-    }
+    if (!found && image->direntOpts == DirEntDontCreate)
+      goto Done;
 
     /* Clear the empty directory entries */
     memset (&dirent[slot], 0xE5, (au * 8 - slot) * sizeof(*dirent));
@@ -2048,10 +2023,8 @@ WriteCpmImage (const struct Filename* name,
     if (slot >= 8 * au ||
         length > (8 * au - slot) * au / 2 * 16 * 128 ||
         length > blocksfree * au * 128) {
-      free (dirent);
-      free (allocated);
-      free (trans);
-      return WrNoSpace;
+      status = WrNoSpace;
+      goto Done;
     }
   }
 
@@ -2110,10 +2083,13 @@ WriteCpmImage (const struct Filename* name,
       memcpy (trans[d], &dirent[d * 8], 8 * sizeof (*dirent));
   }
 
+  status = WrOK;
+
+ Done:
   free (dirent);
   free (allocated);
   free (trans);
-  return WrOK;
+  return status;
 }
 
 /** Read and convert a disk image in C128 CP/M format
@@ -2133,6 +2109,9 @@ ReadCpmImage (FILE* file,
   byte_t** trans;
   unsigned au; /* allocation unit size */
   unsigned sectors; /* number of disk sectors */
+  enum RdStatus status;
+
+  (void) filename; /* unused */
 
   /* determine disk image type from its length */
   {
@@ -2248,6 +2227,7 @@ ReadCpmImage (FILE* file,
       /* Read the file */
       {
         byte_t* curr, *buf = malloc (length);
+        enum WrStatus wrStatus;
 
         if (!buf) {
           (*log) (Warnings, &name, "out of memory");
@@ -2280,23 +2260,21 @@ ReadCpmImage (FILE* file,
         while (length-- && buf[length] == 0x1A);
         length++;
 
-        switch ((*writeCallback) (&name, buf, length)) {
-        case WrOK:
-          break;
-        case WrNoSpace:
-          free (buf);
-          free (image.buf);
-          free (trans);
-          return RdNoSpace;
-        case WrFail:
-        default:
-          free (buf);
-          free (image.buf);
-          free (trans);
-          return RdFail;
-        }
-
+        wrStatus = (*writeCallback) (&name, buf, length);
         free (buf);
+
+        switch (wrStatus) {
+        case WrOK:
+          goto FileDone;
+        case WrNoSpace:
+          status = RdNoSpace;
+          goto Done;
+        case WrFail:
+        case WrFileExists:
+          break;
+        }
+        status = RdFail;
+        goto Done;
       }
 
     FileDone:
@@ -2304,10 +2282,12 @@ ReadCpmImage (FILE* file,
     }
   }
 
+  status = RdOK;
+ Done:
   free (image.buf);
   free (trans);
 
-  return RdOK;
+  return status;
 }
 
 /** Write to an image in CBM DOS format
@@ -2404,7 +2384,7 @@ WriteImage (const struct Filename* name,
       return WrNoSpace;
 
     if (dirent->type) {
-      if (image->direntOpts == DirEntOnlyCreate)
+      if (image->direntOpts == DirEntUniqCreate)
         return WrFileExists;
 
       /* delete the old file */
@@ -2536,7 +2516,7 @@ WriteImage (const struct Filename* name,
     return WrNoSpace;
 
   if (dirent->type) {
-    if (image->direntOpts == DirEntOnlyCreate)
+    if (image->direntOpts == DirEntUniqCreate)
       return WrFileExists;
 
     /* delete the old file */
@@ -2645,6 +2625,9 @@ ReadImage (FILE* file,
 {
   const struct DiskGeometry* geom = 0;
   struct Image image;
+  enum RdStatus status = RdFail;
+
+  (void) filename; /* unused */
 
   /* determine disk image type from its length */
   {
@@ -2694,8 +2677,7 @@ ReadImage (FILE* file,
 
     if (1 != fread (image.buf, length, 1, file)) {
       (*log) (Errors, 0, "fread: %s", strerror(errno));
-      free (image.buf);
-      return RdFail;
+      goto Done;
     }
   }
 
@@ -2707,15 +2689,12 @@ ReadImage (FILE* file,
     if (!(block = mapInode (&directory, &image, image.dirtrack, 0, log, 0))) {
       (*log) (Errors, 0, "Could not read the directory on track %u.",
               image.dirtrack);
-      free (image.buf);
-      return RdFail;
+      goto ReadDone;
     }
 
     if (block < geom->BAMblocks) {
       (*log) (Errors, 0, "Directory too short.");
-      free (image.buf);
-      free (directory);
-      return RdFail;
+      goto ReadDone;
     }
 
     /* Traverse through the directory */
@@ -2725,6 +2704,7 @@ ReadImage (FILE* file,
       struct Filename name;
 
       for (i = 0; i < 256 / sizeof (struct DirEnt); i++) {
+        enum WrStatus wrStatus;
         struct DirEnt* dirent = &((struct DirEnt*) directory[block])[i];
 
         memcpy (name.name, dirent->name, 16);
@@ -2812,16 +2792,14 @@ ReadImage (FILE* file,
 
           if (!(buf = calloc ((2U + dirent->isVLIR) * 254U + length, 1))) {
             (*log) (Errors, &name, "Out of memory");
-            free (image.buf);
-            free (directory);
-            return RdFail;
+            goto ReadDone;
           }
 
           /* set the Convert header data */
           memcpy (&buf[0], &dirent->type, length = sizeof (struct DirEnt) - 2);
           memcpy (&buf[length], cvt, sizeof cvt); length += sizeof cvt;
-          /* clear the track/sector information from the header */
-          buf[1] = buf[2] = buf[0x13] = buf[0x14] = 0;
+          /* the track/sector information was already cleared by calloc() */
+          /* buf[1] = buf[2] = buf[0x13] = buf[0x14] = 0; */
 
           /* copy the info block */
           memcpy (&buf[254], &info[2], 254);
@@ -2895,23 +2873,22 @@ ReadImage (FILE* file,
             free (b);
           }
 
-          switch ((*writeCallback) (&name, buf, length)) {
+          wrStatus = (*writeCallback) (&name, buf, length);
+          free (buf);
+
+          switch (wrStatus) {
           case WrOK:
             continue;
           case WrNoSpace:
-            free (buf);
-            free (image.buf);
-            free (directory);
-            return RdNoSpace;
+            status = RdNoSpace;
+            /* fall through */
           case WrFail:
-          default:
-            free (buf);
-            free (image.buf);
-            free (directory);
-            return RdFail;
-          notGEOS:
-            (*log) (Warnings, &name, "not a valid GEOS file");
+          case WrFileExists:
+            break;
           }
+          goto ReadDone;
+        notGEOS:
+          (*log) (Warnings, &name, "not a valid GEOS file");
         }
         switch (name.type) {
           byte_t* buf;
@@ -2932,34 +2909,30 @@ ReadImage (FILE* file,
               dirent->blocksLow + ((unsigned) dirent->blocksHigh << 8))
             (*log) (Warnings, &name, "invalid block count");
 
-          switch ((*writeCallback) (&name, buf, length)) {
+          wrStatus = (*writeCallback) (&name, buf, length);
+          free (buf);
+
+          switch (wrStatus) {
           case WrOK:
             break;
           case WrNoSpace:
-            free (buf);
-            free (image.buf);
-            free (directory);
-            return RdNoSpace;
+            status = RdNoSpace;
+            /* fall through */
           case WrFail:
           default:
-            free (buf);
-            free (image.buf);
-            free (directory);
-            return RdFail;
+            goto ReadDone;
           }
 
-          free (buf);
           break;
 
         case CBM:
-          if(image.type == Im1581) {
+          if (image.type == Im1581) {
             (*log) (Errors, &name, "skipping partition");
             /* TODO: Recurse... */
             break;
           }
 
-          /* Fall through (CBM type only supported by the 1581) */
-
+          /* fall through */
         default:
           if (dirent->type)
             (*log) (Errors, &name, "unknown file type $%02x, skipping",
@@ -2967,15 +2940,19 @@ ReadImage (FILE* file,
         }
       }
 
-      if (!((struct DirEnt*) directory[block])->nextTrack)
+      if (!((struct DirEnt*) directory[block])->nextTrack) {
+        status = RdOK;
         break;
+      }
     }
 
+  ReadDone:
     free (directory);
   }
 
+ Done:
   free (image.buf);
-  return RdOK;
+  return status;
 }
 
 /** Open an existing disk image or create a new one.
@@ -3007,7 +2984,8 @@ OpenImage (const char* filename,
   if (!(*image = calloc (1, sizeof (**image))))
     return ImFail;
 
-  if (!((*image)->buf = malloc (geom->blocks * 256))) {
+  if (!((*image)->buf = malloc (geom->blocks * 256)) ||
+      !((*image)->name = malloc (strlen (filename) + 1))) {
   Failed:
     free ((*image)->name);
     free ((*image)->buf);
@@ -3016,9 +2994,6 @@ OpenImage (const char* filename,
     return ImFail;
   }
 
-  if (!((*image)->name = malloc (strlen (filename) + 1)))
-    goto Failed;
-
   strcpy ((char*)(*image)->name, filename);
   (*image)->type = type;
   (*image)->direntOpts = direntOpts;
