@@ -5,7 +5,7 @@
  */
 
 /*
-** Copyright © 1993‒1998,2001,2003,2006,2021‒2022 Marko Mäkelä
+** Copyright © 1993‒1998,2001,2003,2006,2021‒2022,2024 Marko Mäkelä
 **
 **     This program is free software; you can redistribute it and/or modify
 **     it under the terms of the GNU General Public License as published by
@@ -63,7 +63,12 @@ enum ChangeDisks
 };
 
 /** The default disk image changing policy */
-enum ChangeDisks changeDisks = Sometimes;
+static enum ChangeDisks changeDisks = Sometimes;
+
+/** Whether io allow duplicate file names */
+bool allowDuplicates = false;
+/** Whether io ignore duplicate file names */
+static bool ignoreDuplicates = false;
 
 /** Call-back function for diagnostic output
  * @param verbosity     the verbosity level
@@ -105,6 +110,17 @@ writeLog (enum Verbosity verbosity,
   }
 }
 
+/** Report a duplicate file name
+ * @param name          native (PETSCII) name of the file
+ * @retval WrOk  (always)
+ */
+static enum WrStatus
+reportDuplicateName (const struct Filename* name)
+{
+  writeLog (Warnings, name, "skipping file with non-unique name");
+  return WrOK;
+}
+
 /** Write a file
  * @param name          native (PETSCII) name of the file
  * @param data          the contents of the file
@@ -135,11 +151,13 @@ writeFile (const struct Filename* name,
       writeLog (Errors, name, "Write failed!");
       return WrFail;
     case WrFileExists:
+      if (ignoreDuplicates)
+        return reportDuplicateName (name);
       if (changeDisks < Always) {
         writeLog (Errors, name, "non-unique file name!");
         return WrFileExists;
       }
-      /* non-unique file name, fall through */
+      /* fall through */
     case WrNoSpace:
       if (changeDisks < Sometimes) {
         writeLog (Errors, name, "out of space!");
@@ -155,16 +173,17 @@ writeFile (const struct Filename* name,
         unsigned char* c;
       case ImNoSpace:
         writeLog (Errors, name, "out of space");
-        free (image->name);
-        free (image);
-        image = 0;
-        return WrNoSpace;
+        status = WrNoSpace;
+        goto ImageFailed;
       case ImFail:
         writeLog (Errors, name, "failed");
+      ImageFail:
+        status = WrFail;
+      ImageFailed:
         free (image->name);
         free (image);
         image = 0;
-        return WrFail;
+        return status;
       case ImOK:
         writeLog (Everything, name, "wrote old image \"%s\"", image->name);
         /*
@@ -188,10 +207,7 @@ writeFile (const struct Filename* name,
         if (c < image->name) {
         notUnique:
           writeLog (Errors, name, "Could not generate unique image file name");
-          free (image->name);
-          free (image);
-          image = 0;
-          return WrFail;
+          goto ImageFail;
         }
 
         writeLog (Everything, name, "Continuing to image \"%s\"...",
@@ -207,32 +223,33 @@ writeFile (const struct Filename* name,
 
           imstatus = OpenImage (filename, &image, type, direntOpts);
 
-          free (filename);
-
           switch (imstatus) {
           case ImOK:
             status = (*writeImageFunc) (name, data, length, image, writeLog);
 
             if (status == WrOK)
               writeLog (Everything, name, "OK, wrote %u bytes to image \"%s\"",
-                        length, image->name);
+                        length, filename);
             else
               writeLog (Errors, name, "%s while writing to \"%s\", giving up.",
                         status == WrNoSpace ? "out of space" :
                         status == WrFileExists ? "duplicate file name" :
                         "failed",
-                        image->name);
-            return status;
-
+                        filename);
+            break;
           case ImNoSpace:
             writeLog (Errors, name,
-                      "out of space while creating image \"%s\"", image->name);
-            return WrNoSpace;
+                      "out of space while creating image \"%s\"", filename);
+            status = WrNoSpace;
+            break;
           default:
             writeLog (Errors, name, "failed while creating image \"%s\"",
-                      image->name);
-            return WrFail;
+                      filename);
+            status = WrFail;
           }
+
+          free (filename);
+          return status;
         }
       }
     }
@@ -248,6 +265,8 @@ writeFile (const struct Filename* name,
       writeLog (Errors, name, "Write failed!");
       return WrFail;
     case WrFileExists:
+      if (ignoreDuplicates)
+        return reportDuplicateName (name);
       writeLog (Errors, name, "non-unique file name!");
       return WrFileExists;
     case WrNoSpace:
@@ -357,6 +376,23 @@ main (int argc, char** argv)
         opts++;
         break;
 
+      case 'o':
+        switch (opts[1]) {
+        case '0':
+        case '1':
+          ignoreDuplicates = opts[1] != '0';
+          allowDuplicates = false;
+          break;
+        case '2':
+          ignoreDuplicates = false;
+          allowDuplicates = true;
+          break;
+        default:
+          goto Usage;
+        }
+        opts++;
+        break;
+
       case 'n':
         readFunc = ReadNative;
         break;
@@ -439,7 +475,11 @@ main (int argc, char** argv)
           argv++;
 
           {
-            enum DirEntOpts dopts = DirEntOnlyCreate;
+            enum DirEntOpts dopts = DirEntUniqCreate;
+            if (opts[1] == 'd') {
+              dopts = DirEntDupCreate;
+              opts++;
+            }
             if (opts[1] == 'o') {
               dopts = DirEntFindOrCreate;
               opts++;
@@ -475,17 +515,22 @@ main (int argc, char** argv)
            "         -L archive.lnx: Output files in Lynx format.\n"
            "         -C archive.c2n: Output files in Commodore C2N format.\n"
            "         -D4 imagefile: Write to a 1541 disk image.\n"
+           "         -D4d imagefile: Ditto, allowing duplicate file names.\n"
            "         -D4o imagefile: Ditto, overwriting existing files.\n"
-           "         -D7[o] imagefile: Write to a 1571 disk image.\n"
-           "         -D8[o] imagefile: Write to a 1581 disk image.\n"
-           "         -M4[o] imagefile: Write to a 1541 CP/M disk image.\n"
-           "         -M7[o] imagefile: Write to a 1571 CP/M disk image.\n"
-           "         -M8[o] imagefile: Write to a 1581 CP/M disk image.\n"
+           "         -D7[do] imagefile: Write to a 1571 disk image.\n"
+           "         -D8[do] imagefile: Write to a 1581 disk image.\n"
+           "         -M4[do] imagefile: Write to a 1541 CP/M disk image.\n"
+           "         -M7[do] imagefile: Write to a 1571 CP/M disk image.\n"
+           "         -M8[do] imagefile: Write to a 1581 CP/M disk image.\n"
            "\n"
            "         -i2: Switch disk images on out of space or duplicate file name.\n"
            "         -i1: Switch disk images on out of space.\n"
            "         -i0: Never switch disk images.\n"
            "\n"
+           "         -o0: Detect files with duplicate names\n"
+           "         -o1: Ignore files with duplicate names\n"
+           "         -o2: Allow files with duplicate names\n"
+           "\n"
            "         -n: input files in native format.\n"
            "         -p: input files in PC64 format.\n"
            "         -a: input files in ARC/SDA format.\n"
